<?php
namespace FiloBlu\Flow\Model\Parser;

abstract class AbstractParser{
    public $map;
    private $attribute_set_id;
    private $configurable_attributes;
    private $websites;
    private $stores;
    protected $_channel;

    abstract public function process($resource);

    /**
    * @var Monolog\Logger
    */
    protected $_logger;

    public function __construct(
             \FiloBlu\Flow\Helper\LoggerProvider $_loggerProvider
    ) {
        $this->_logger = $_loggerProvider->getLogger();

    }

    public function setConfig($config)
    {

        return $this;

    }

    public function setChannel($channel) {
        $this->_channel = $channel;
    }

    public function getChannel() {
        return $this->_channel;
    }

    /*
     * Validate single field applying Validators
     */
    protected function _validate($value,$validators,$field){
        foreach($validators as $validator){
            $validatorClass = 'FiloBlu\Bouncer\Model\Parser\Validators' . "\\" . ucfirst($validator);
            $validator = new $validatorClass();
            if(!$validator->isValid($value)){
               return false;
            }
        }
        return true;
    }
    /*
     * Validate single field applying Validators
     */
    protected function _filter($value,$filters,$data){
        if(count($filters) > 0) {
            foreach ($filters as $filterData) {
                $filterClass = 'FiloBlu\Flow\Model\Filters' . "\\" . ucfirst($filterData['class']);
                $filter =  \Magento\Framework\App\ObjectManager::getInstance()->create($filterClass);
                if(method_exists($filter,"setRow")) {
                    $filter->setRow($data);
                }
                $value = $filter->filter($value,$filterData['args']);
            }
        }
        return $value;
    }

    public function loadMap($maplocation){
        $map =  \Magento\Framework\App\ObjectManager::getInstance()->create('FiloBlu\Flow\Model\Map');
        $map->load($maplocation);
        if($map->initialized){
            $this->map = $map;
            return true;
        }
        return false;
    }

    /*
     * Parse Data
     *
     */
    public function parse($data,$key_mode = false){
        $this->_logger->info('FiloBlu\Flow\Model\Parser\AbstractParser::parse(data = "' . print_r($data, true) . '", $key_mode = ' . $key_mode . ')');
        if(!$key_mode && !$this->map->checkData($data)){
            throw new \Exception('Data don\'t match mapping count ' . count($data));
        }

        $channel = $this->getChannel();

        $fields = $this->map->getFields();
        $parsed_data = array();

        foreach($fields as $i => $field){
            $validators = array();
            $filters = array();
            $fieldMap = $this->map->get($field);
            $key = ($key_mode) ? (isset($fieldMap['from'])?$fieldMap['from']:$i) : $i;
            if(array_key_exists(trim($key), $data)) {
                $fieldData = $data[trim($key)];
                if (!isset($fieldMap['skip'])) {
                    if (isset($fieldMap['validators'])) {
                        $validators = $fieldMap['validators'];
                    }
                    if (isset($fieldMap['filters'])) {
                        $filters = $fieldMap['filters'];
                    }
                    if ($this->_validate($fieldData, $validators, $field)) {
                        $value = $this->_filter($fieldData, $filters, $data);
                        $parsed_data[$field] = $value;
                    } else {
                        throw new \Exception("$value is not Valid for $field");
                    }
                }
            }

        }

        if (method_exists($channel, 'insertData') && !is_null($channel)) {

            try {
                $this->_logger->info('FiloBlu\Flow\Model\Parser\AbstractParser::parse, parsed_data = "' . print_r($parsed_data, true));
                $processed = $channel->insertData($parsed_data);
            }
            catch (\Exception $e) {
                throw new \Exception($e->getMessage());
            }

        }
        else
            throw new \Exception('no method insertData for class ' . get_class($channel));


    }


}
